package tdspp.util;

import java.io.*;

/**
* ArmyResources class represents the resources used by a cell on the map.
*
* The basic assumption is that there can be at the most one unit of all the
resources
* in a cell. So all we need to store is either a 1 or 0 for all the types of
resources.
*
* <table border=0>
* <tr> <td style="font-weight: bold; font-size: smaller"> Belongs to </td>
<td> TDS++ </td>
* <tr> <td style="font-weight: bold; font-size: smaller"> Properties </td>
<td> Utility </td>
* </table>
*
* @author 	TDS++ team, USC-CSE
* @version 	1.0 , 23 May 2002
*
*/

public class ArmyResources implements Cloneable, Serializable  {
	/**
	 * Resources are stored as a bit sequence. The first (upper) 16 bits give the
friendly resources and
	 * second (lower) 16 bits give the enemy resources known. In both these 16-bit
sequences, the first one represents
	 * the tank unit, second represents the infantry unit and the third one
gives the mine.
	 *
	*/
	public int resources;

	/**
	 * The state of ammunition available in a particular cell. The state is
represented
	 * as a float value which ranges from 0 to 1. 1 means full availability.
	 *
	*/
	public float stateAmmunition;

	/**
	 * The state of food material available in the cell.
	 *
	*/
	public float stateFood;

	/**
	 * The state of population available in the cell.
	 *
	*/
	public float statePopulation;

	/**
	 * Constructor
	 *
	 */
	public void ArmyResources() {
		this.resources = 0;
		this.stateAmmunition = (float)0.0;
		this.stateFood = (float)0.0;
		this.statePopulation = (float)0.0;
	}

	/**
	 * set stateAmmunition
	 */
	public void setStateAmmunition (float newValue) {
		this.stateAmmunition = newValue;
	}

	/**
	 * get stateAmmunition
	 */
	public float getStateAmmunition () {
		return this.stateAmmunition;
	}


	/**
	 * set stateFood
	 */
	public void setStateFood(float newValue) {
		this.stateFood = newValue;
	}

	/**
	 * get stateFood
	 */
	public float getStateFood() {
		return this.stateFood;
	}


	/**
	 * set statePopulation
	 */
	public void setStatePopulation (float newValue) {
		this.statePopulation = newValue;
	}

	/**
	 * get statePopulation
	 */
	public float getStatePopulation () {
		return this.statePopulation;
	}

	/**
	 * set empty
	 */
	public void setEmpty() {
		this.resources = 0;
	}

	/**
	 * set mine field
	 */
	public void setMineField() {
		this.resources |= tdspp.util.Constants.ARMYRESOURCES_MINEFIELD;
	}

	/**
	 * set tank
	 */
	public void setTank() {
		this.resources |= tdspp.util.Constants.ARMYRESOURCES_TANKFIELD;
	}

	/**
	 * set infantry
	 */
	public void setInfantry() {
		this.resources |= tdspp.util.Constants.ARMYRESOURCES_INFANTRY;
	}

	/**
	 * set enemy tank
	 */
	public void setEnemyTank() {
		this.resources |= tdspp.util.Constants.ARMYRESOURCES_ENEMYTANK;
	}

	/**
	 * set enemy infantry
	 */
	public void setEnemyInfantry() {
		this.resources |= tdspp.util.Constants.ARMYRESOURCES_ENEMYINFT;
	}


	/**
	 * remove mine field
	 */
	public void removeMineField() {
		this.resources &= ~tdspp.util.Constants.ARMYRESOURCES_MINEFIELD;
	}

	/**
	 * remove tank
	 */
	public void removeTank() {
		this.resources &= ~tdspp.util.Constants.ARMYRESOURCES_TANKFIELD;
	}

	/**
	 * remove infantry
	 */
	public void removeInfantry() {
		this.resources &= ~tdspp.util.Constants.ARMYRESOURCES_INFANTRY;
	}

	/**
	 * remove enemy tank
	 */
	public void removeEnemyTank() {
		this.resources &= ~tdspp.util.Constants.ARMYRESOURCES_ENEMYTANK;
	}

	/**
	 * remove enemy infantry
	 */
	public void removeEnemyInfantry() {
		this.resources &= ~tdspp.util.Constants.ARMYRESOURCES_ENEMYINFT;
	}

	/**
	 * is empty
	 */
	public boolean isEmpty() {
		return (this.resources == 0);
	}

	/**
	 * is mine field
	 */
	public boolean isMineField() {
		return ((this.resources & tdspp.util.Constants.ARMYRESOURCES_MINEFIELD) ==
tdspp.util.Constants.ARMYRESOURCES_MINEFIELD);
	}

	/**
	 * is tank
	 */
	public boolean isTank() {
		return ((this.resources & tdspp.util.Constants.ARMYRESOURCES_TANKFIELD) ==
tdspp.util.Constants.ARMYRESOURCES_TANKFIELD);
	}

	/**
	 * is infantry
	 */
	public boolean isInfantry() {
		return ((this.resources & tdspp.util.Constants.ARMYRESOURCES_INFANTRY) ==
tdspp.util.Constants.ARMYRESOURCES_INFANTRY);
	}

	/**
	 * is enemy tank
	 */
	public boolean isEnemyTank() {
		return ((this.resources & tdspp.util.Constants.ARMYRESOURCES_ENEMYTANK) ==
tdspp.util.Constants.ARMYRESOURCES_ENEMYTANK);
	}

	/**
	 * is enemy infantry
	 */
	public boolean isEnemyInfantry() {
		return ((this.resources & tdspp.util.Constants.ARMYRESOURCES_ENEMYINFT) ==
tdspp.util.Constants.ARMYRESOURCES_ENEMYINFT);
	}


	public float getTotalState(){
		return (stateFood + statePopulation + stateAmmunition)/3;
	}

	/**
	 * cloning function
	 */
	public Object clone() {
		Object o = null;
		try {
			o = super.clone();
		} catch(CloneNotSupportedException e) {
			e.printStackTrace(System.err);
		}
		return o;
	}

	public String toString() {
			String data;
			data = "Resources : ";
			if(isInfantry())
				data += "Infantry ,";
			if(isEnemyInfantry())
				data += "EnemyInfantry ,";
			if(isTank())
				data += "Tank ,";
			if(isEnemyTank())
				data += "EnemyTank ,";
			if(isMineField())
				data += "MineField";

			return data;
	}

	public String toXML(){
		String output = "";
		output += "<ArmyResources type=class>";
		output += "<resources type=Integer>" + resources + "</resources>";
		output += "<stateFood type=Float>" + stateFood  + "</stateFood>";
		output += "<stateAmmunition type=Float>" + stateAmmunition + "</stateAmmunition>";
		output += "<statePopulation type=Float>" + statePopulation + "</statePopulation>";
		output += "</ArmyResources>";
		return output;
	}

}

